<?php

namespace Drupal\keycrm\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class KeyCRMSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'keycrm_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['keycrm.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('keycrm.settings');

    $form['enable_sync'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable order synchronization'),
      '#default_value' => $config->get('enable_sync'),
    ];

    $form['environment'] = [
      '#type' => 'select',
      '#title' => $this->t('Environment'),
      '#options' => [
        'local' => $this->t('Local'),
        'dev' => $this->t('Development'),
        'prod' => $this->t('Production'),
      ],
      '#default_value' => $config->get('environment') ?? 'dev',
      '#required' => TRUE,
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
    ];

    // Payment methods.
    $storage = $form_state->getStorage();
    if (!isset($storage['payment_types'])) {
      $storage['payment_types'] = $config->get('payment_types') ?: [];
      $form_state->setStorage($storage);
    }
    $payment_types = $storage['payment_types'];

    $form['payment_wrapper'] = [
      '#type' => 'container',
      '#prefix' => '<div id="payment-wrapper">',
      '#suffix' => '</div>',
    ];
    $form['payment_wrapper']['payment_types'] = [
      '#type' => 'table',
      '#header' => [$this->t('Payment method'), $this->t('ID'), $this->t('Operations')],
      '#empty' => $this->t('No payment types added yet.'),
    ];

    foreach ($payment_types as $key => $value) {
      $form['payment_wrapper']['payment_types'][$key]['method'] = [
        '#type' => 'textfield',
        '#default_value' => $key,
      ];
      $form['payment_wrapper']['payment_types'][$key]['id'] = [
        '#type' => 'textfield',
        '#default_value' => $value,
      ];
      $form['payment_wrapper']['payment_types'][$key]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#name' => 'remove_payment_' . $key,
        '#submit' => [[get_class($this), 'removePaymentSubmit']],
        '#ajax' => [
          'callback' => [get_class($this), 'ajaxRefreshPayments'],
          'wrapper' => 'payment-wrapper',
        ],
      ];
    }

    $form['payment_wrapper']['add_payment'] = [
      '#type' => 'details',
      '#title' => $this->t('Add payment method'),
      '#open' => TRUE,
    ];

    $form['payment_wrapper']['add_payment']['payment_new_method'] = [
      '#type' => 'textfield',
      '#placeholder' => 'mashine_name in Drupal',
      '#title' => $this->t('Method'),
    ];
    $form['payment_wrapper']['add_payment']['payment_new_id'] = [
      '#type' => 'textfield',
      '#placeholder' => 'ID in KeyCRM',
      '#title' => $this->t('ID'),
    ];
    $form['payment_wrapper']['add_payment']['add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add payment method'),
      '#name' => 'add_payment',
      '#submit' => [[get_class($this), 'addPaymentAjaxSubmit']],
      '#ajax' => [
        'callback' => [get_class($this), 'ajaxRefreshPayments'],
        'wrapper' => 'payment-wrapper',
      ],
    ];


    // Shipping methods.
    $storage = $form_state->getStorage();
    if (!isset($storage['shipping_types'])) {
      $storage['shipping_types'] = $config->get('shipping_types') ?: [];
      $form_state->setStorage($storage);
    }
    $shipping_types = $storage['shipping_types'];

    $form['shipping_wrapper'] = [
      '#type' => 'container',
      '#prefix' => '<div id="shipping-wrapper">',
      '#suffix' => '</div>',
    ];
    $form['shipping_wrapper']['shipping_types'] = [
      '#type' => 'table',
      '#header' => [$this->t('Shipping method'), $this->t('ID'), $this->t('Operations')],
      '#empty' => $this->t('No shipping types added yet.'),
    ];

    foreach ($shipping_types as $key => $value) {
      $form['shipping_wrapper']['shipping_types'][$key]['method'] = [
        '#type' => 'textfield',
        '#default_value' => $key,
      ];
      $form['shipping_wrapper']['shipping_types'][$key]['id'] = [
        '#type' => 'textfield',
        '#default_value' => $value,
      ];
      $form['shipping_wrapper']['shipping_types'][$key]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#name' => 'remove_shipping_' . $key,
        '#submit' => [[get_class($this), 'removeShippingSubmit']],
        '#ajax' => [
          'callback' => [get_class($this), 'ajaxRefreshShipping'],
          'wrapper' => 'shipping-wrapper',
        ],
      ];
    }

    $form['shipping_wrapper']['add_shipping'] = [
      '#type' => 'details',
      '#title' => $this->t('Add shipping method'),
      '#open' => TRUE,
    ];
    $form['shipping_wrapper']['add_shipping']['shipping_new_method'] = [
      '#type' => 'textfield',
      '#placeholder' => 'mashine_name in Drupal',
      '#title' => $this->t('Method'),
    ];
    $form['shipping_wrapper']['add_shipping']['shipping_new_id'] = [
      '#type' => 'textfield',
      '#placeholder' => 'ID in KeyCRM',
      '#title' => $this->t('ID'),
    ];
    $form['shipping_wrapper']['add_shipping']['add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add shipping method'),
      '#name' => 'add_shipping',
      '#submit' => [[get_class($this), 'addShippingAjaxSubmit']],
      '#ajax' => [
        'callback' => [get_class($this), 'ajaxRefreshShipping'],
        'wrapper' => 'shipping-wrapper',
      ],
    ];


    return parent::buildForm($form, $form_state);
  }


  public static function addPaymentAjaxSubmit(array &$form, FormStateInterface $form_state) {
    $method = trim($form_state->getValue('payment_new_method'));
    $id = trim($form_state->getValue('payment_new_id'));
    $storage = $form_state->getStorage();
    $payment_types = $storage['payment_types'] ?? [];

    if (!empty($method) && is_numeric($id)) {
      $payment_types[$method] = $id;
      $storage['payment_types'] = $payment_types;
      $form_state->setStorage($storage);

      $form_state->setValue('new_method', '');
      $form_state->setValue('new_id', '');
    }

    $form_state->setRebuild();
  }

  public static function removePaymentSubmit(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $name = $triggering_element['#name'];

    if (preg_match('/^remove_payment_(.+)$/', $name, $matches)) {
      $method = $matches[1];
      $payment_types = $form_state->get('payment_types') ?? [];
      unset($payment_types[$method]);
      $form_state->set('payment_types', $payment_types);
      $form_state->setRebuild();
    }
  }

  public static function ajaxRefreshPayments(array &$form, FormStateInterface $form_state) {
    return $form['payment_wrapper'];
  }

  public static function addShippingAjaxSubmit(array &$form, FormStateInterface $form_state) {
    $method = trim($form_state->getValue('shipping_new_method'));
    $id = trim($form_state->getValue('shipping_new_id'));
    $storage = $form_state->getStorage();
    $shipping_types = $storage['shipping_types'] ?? [];

    if (!empty($method) && is_numeric($id)) {
      $shipping_types[$method] = $id;
      $storage['shipping_types'] = $shipping_types;
      $form_state->setStorage($storage);

      $form_state->setValue('new_method', '');
      $form_state->setValue('new_id', '');
    }

    $form_state->setRebuild();
  }

  public static function removeShippingSubmit(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $name = $triggering_element['#name'];

    if (preg_match('/^remove_shipping_(.+)$/', $name, $matches)) {
      $method = $matches[1];
      $shipping_types = $form_state->getStorage()['shipping_types'] ?? [];
      unset($shipping_types[$method]);
      $storage = $form_state->getStorage();
      $storage['shipping_types'] = $shipping_types;
      $form_state->setStorage($storage);
      $form_state->setRebuild();
    }
  }

  public static function ajaxRefreshShipping(array &$form, FormStateInterface $form_state) {
    return $form['shipping_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Preparing payment data for saving.
    $payment_types_input = $form_state->get('payment_types') ?? [];
    $payment_types = [];
    foreach ($payment_types_input as $key => $value) {
      if ($key === '__new' && !empty($value) && strpos($value, '|') !== FALSE) {
        [$method, $id] = array_map('trim', explode('|', $value));
        $payment_types[$method] = $id;
      } elseif ($key !== '__new' && is_numeric($value)) {
        $payment_types[$key] = $value;
      }
    }

    // Preparing shipping data for saving.
    $shipping_types_input = $form_state->getStorage()['shipping_types'] ?? [];
    $shipping_types = [];
    foreach ($shipping_types_input as $key => $value) {
      if ($key === '__new' && !empty($value) && strpos($value, '|') !== FALSE) {
        [$method, $id] = array_map('trim', explode('|', $value));
        $shipping_types[$method] = $id;
      } elseif ($key !== '__new' && is_numeric($value)) {
        $shipping_types[$key] = $value;
      }
    }

    $this->config('keycrm.settings')
      ->set('enable_sync', $form_state->getValue('enable_sync'))
      ->set('environment', $form_state->getValue('environment'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('payment_types', $payment_types)
      ->set('shipping_types', $shipping_types)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
